@@ -15,7 +15,7 @@
 #include "global_variables.h"
 #include "c_util.h"
 
-void init_unicorn_httpdate(void);
+void init_unicorn_httpdate(VALUE mark_ary);
 
 #define UH_FL_CHUNKED  0x1
 #define UH_FL_HASBODY  0x2
@@ -28,6 +28,7 @@ void init_unicorn_httpdate(void);
 #define UH_FL_HASHEADER 0x100
 #define UH_FL_TO_CLEAR 0x200
 #define UH_FL_RESSTART 0x400 /* for check_client_connection */
+#define UH_FL_HIJACK 0x800
 
 /* all of these flags need to be set for keepalive to be supported */
 #define UH_FL_KEEPALIVE (UH_FL_KAVERSION | UH_FL_REQEOF | UH_FL_HASHEADER)
@@ -286,12 +287,12 @@ static void write_value(struct http_parser *hp,
 /** Machine **/
 
 
-#line 393 "unicorn_http.rl"
+#line 394 "unicorn_http.rl"
 
 
 /** Data **/
 
-#line 295 "unicorn_http.c"
+#line 296 "unicorn_http.c"
 static const int http_parser_start = 1;
 static const int http_parser_first_final = 122;
 static const int http_parser_error = 0;
@@ -302,7 +303,7 @@ static const int http_parser_en_Trailers = 114;
 static const int http_parser_en_main = 1;
 
 
-#line 397 "unicorn_http.rl"
+#line 398 "unicorn_http.rl"
 
 static void http_parser_init(struct http_parser *hp)
 {
@@ -315,12 +316,12 @@ static void http_parser_init(struct http_parser *hp)
   hp->len.content = 0;
   hp->cont = Qfalse; /* zero on MRI, should be optimized away by above */
   
-#line 319 "unicorn_http.c"
+#line 320 "unicorn_http.c"
 	{
 	cs = http_parser_start;
 	}
 
-#line 409 "unicorn_http.rl"
+#line 410 "unicorn_http.rl"
   hp->cs = cs;
 }
 
@@ -348,7 +349,7 @@ http_parser_execute(struct http_parser *hp, char *buffer, size_t len)
     goto skip_chunk_data_hack;
   }
   
-#line 352 "unicorn_http.c"
+#line 353 "unicorn_http.c"
 	{
 	if ( p == pe )
 		goto _test_eof;
@@ -383,14 +384,14 @@ st0:
 cs = 0;
 	goto _out;
 tr0:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st2;
 st2:
 	if ( ++p == pe )
 		goto _test_eof2;
 case 2:
-#line 394 "unicorn_http.c"
+#line 395 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr3;
 		case 33: goto st49;
@@ -416,14 +417,14 @@ case 2:
 		goto st49;
 	goto st0;
 tr3:
-#line 298 "unicorn_http.rl"
+#line 299 "unicorn_http.rl"
 	{ request_method(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st3;
 st3:
 	if ( ++p == pe )
 		goto _test_eof3;
 case 3:
-#line 427 "unicorn_http.c"
+#line 428 "unicorn_http.c"
 	switch( (*p) ) {
 		case 42: goto tr5;
 		case 47: goto tr6;
@@ -432,21 +433,21 @@ case 3:
 	}
 	goto st0;
 tr5:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st4;
 st4:
 	if ( ++p == pe )
 		goto _test_eof4;
 case 4:
-#line 443 "unicorn_http.c"
+#line 444 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr8;
 		case 35: goto tr9;
 	}
 	goto st0;
 tr8:
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -464,23 +465,23 @@ tr8:
   }
 	goto st5;
 tr42:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 318 "unicorn_http.rl"
+#line 319 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st5;
 tr45:
-#line 318 "unicorn_http.rl"
+#line 319 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st5;
 tr49:
-#line 328 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -491,7 +492,7 @@ tr49:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -509,14 +510,14 @@ tr49:
   }
 	goto st5;
 tr55:
-#line 322 "unicorn_http.rl"
-	{MARK(start.query, p); }
 #line 323 "unicorn_http.rl"
+	{MARK(start.query, p); }
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -534,12 +535,12 @@ tr55:
   }
 	goto st5;
 tr59:
-#line 323 "unicorn_http.rl"
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -560,19 +561,19 @@ st5:
 	if ( ++p == pe )
 		goto _test_eof5;
 case 5:
-#line 564 "unicorn_http.c"
+#line 565 "unicorn_http.c"
 	if ( (*p) == 72 )
 		goto tr10;
 	goto st0;
 tr10:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st6;
 st6:
 	if ( ++p == pe )
 		goto _test_eof6;
 case 6:
-#line 576 "unicorn_http.c"
+#line 577 "unicorn_http.c"
 	if ( (*p) == 84 )
 		goto st7;
 	goto st0;
@@ -632,34 +633,34 @@ case 13:
 		goto st13;
 	goto st0;
 tr18:
-#line 327 "unicorn_http.rl"
+#line 328 "unicorn_http.rl"
 	{ http_version(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st14;
 tr26:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st14;
 tr29:
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st14;
 tr36:
-#line 295 "unicorn_http.rl"
-	{ MARK(mark, p); }
 #line 296 "unicorn_http.rl"
+	{ MARK(mark, p); }
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st14;
 tr39:
-#line 296 "unicorn_http.rl"
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st14;
 st14:
 	if ( ++p == pe )
 		goto _test_eof14;
 case 14:
-#line 663 "unicorn_http.c"
+#line 664 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto st15;
 		case 10: goto tr21;
@@ -688,14 +689,14 @@ case 14:
 		goto tr23;
 	goto st0;
 tr25:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st15;
 st15:
 	if ( ++p == pe )
 		goto _test_eof15;
 case 15:
-#line 699 "unicorn_http.c"
+#line 700 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr25;
 		case 10: goto tr26;
@@ -707,14 +708,14 @@ case 15:
 		goto st0;
 	goto tr24;
 tr24:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st16;
 st16:
 	if ( ++p == pe )
 		goto _test_eof16;
 case 16:
-#line 718 "unicorn_http.c"
+#line 719 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr29;
 		case 13: goto tr30;
@@ -727,39 +728,39 @@ case 16:
 		goto st0;
 	goto st16;
 tr19:
-#line 327 "unicorn_http.rl"
+#line 328 "unicorn_http.rl"
 	{ http_version(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st17;
 tr27:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st17;
 tr30:
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st17;
 tr37:
-#line 295 "unicorn_http.rl"
-	{ MARK(mark, p); }
 #line 296 "unicorn_http.rl"
+	{ MARK(mark, p); }
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st17;
 tr40:
-#line 296 "unicorn_http.rl"
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st17;
 st17:
 	if ( ++p == pe )
 		goto _test_eof17;
 case 17:
-#line 758 "unicorn_http.c"
+#line 759 "unicorn_http.c"
 	if ( (*p) == 10 )
 		goto st14;
 	goto st0;
 tr21:
-#line 343 "unicorn_http.rl"
+#line 344 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -780,7 +781,7 @@ tr21:
   }
 	goto st122;
 tr104:
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -796,7 +797,7 @@ tr104:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 343 "unicorn_http.rl"
+#line 344 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -817,14 +818,14 @@ tr104:
   }
 	goto st122;
 tr108:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 318 "unicorn_http.rl"
+#line 319 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
-#line 343 "unicorn_http.rl"
+#line 344 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -845,12 +846,12 @@ tr108:
   }
 	goto st122;
 tr112:
-#line 318 "unicorn_http.rl"
+#line 319 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
-#line 343 "unicorn_http.rl"
+#line 344 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -871,7 +872,7 @@ tr112:
   }
 	goto st122;
 tr117:
-#line 328 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -882,7 +883,7 @@ tr117:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -898,7 +899,7 @@ tr117:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 343 "unicorn_http.rl"
+#line 344 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -919,14 +920,14 @@ tr117:
   }
 	goto st122;
 tr124:
-#line 322 "unicorn_http.rl"
-	{MARK(start.query, p); }
 #line 323 "unicorn_http.rl"
+	{MARK(start.query, p); }
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -942,7 +943,7 @@ tr124:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 343 "unicorn_http.rl"
+#line 344 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -963,12 +964,12 @@ tr124:
   }
 	goto st122;
 tr129:
-#line 323 "unicorn_http.rl"
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -984,7 +985,7 @@ tr129:
       rb_hash_aset(hp->env, g_request_path, str);
     }
   }
-#line 343 "unicorn_http.rl"
+#line 344 "unicorn_http.rl"
 	{
     finalize_header(hp);
 
@@ -1008,10 +1009,10 @@ st122:
 	if ( ++p == pe )
 		goto _test_eof122;
 case 122:
-#line 1012 "unicorn_http.c"
+#line 1013 "unicorn_http.c"
 	goto st0;
 tr105:
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1029,23 +1030,23 @@ tr105:
   }
 	goto st18;
 tr109:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 318 "unicorn_http.rl"
+#line 319 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st18;
 tr113:
-#line 318 "unicorn_http.rl"
+#line 319 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(mark, p), FRAGMENT);
     rb_hash_aset(hp->env, g_fragment, STR_NEW(mark, p));
   }
 	goto st18;
 tr118:
-#line 328 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -1056,7 +1057,7 @@ tr118:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1074,14 +1075,14 @@ tr118:
   }
 	goto st18;
 tr125:
-#line 322 "unicorn_http.rl"
-	{MARK(start.query, p); }
 #line 323 "unicorn_http.rl"
+	{MARK(start.query, p); }
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1099,12 +1100,12 @@ tr125:
   }
 	goto st18;
 tr130:
-#line 323 "unicorn_http.rl"
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1125,25 +1126,25 @@ st18:
 	if ( ++p == pe )
 		goto _test_eof18;
 case 18:
-#line 1129 "unicorn_http.c"
+#line 1130 "unicorn_http.c"
 	if ( (*p) == 10 )
 		goto tr21;
 	goto st0;
 tr23:
-#line 291 "unicorn_http.rl"
-	{ MARK(start.field, p); }
 #line 292 "unicorn_http.rl"
+	{ MARK(start.field, p); }
+#line 293 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st19;
 tr32:
-#line 292 "unicorn_http.rl"
+#line 293 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st19;
 st19:
 	if ( ++p == pe )
 		goto _test_eof19;
 case 19:
-#line 1147 "unicorn_http.c"
+#line 1148 "unicorn_http.c"
 	switch( (*p) ) {
 		case 33: goto tr32;
 		case 58: goto tr33;
@@ -1169,18 +1170,18 @@ case 19:
 		goto tr32;
 	goto st0;
 tr35:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st20;
 tr33:
-#line 294 "unicorn_http.rl"
+#line 295 "unicorn_http.rl"
 	{ hp->s.field_len = LEN(start.field, p); }
 	goto st20;
 st20:
 	if ( ++p == pe )
 		goto _test_eof20;
 case 20:
-#line 1184 "unicorn_http.c"
+#line 1185 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr35;
 		case 10: goto tr36;
@@ -1192,14 +1193,14 @@ case 20:
 		goto st0;
 	goto tr34;
 tr34:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st21;
 st21:
 	if ( ++p == pe )
 		goto _test_eof21;
 case 21:
-#line 1203 "unicorn_http.c"
+#line 1204 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr39;
 		case 13: goto tr40;
@@ -1212,7 +1213,7 @@ case 21:
 		goto st0;
 	goto st21;
 tr9:
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1230,7 +1231,7 @@ tr9:
   }
 	goto st22;
 tr50:
-#line 328 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -1241,7 +1242,7 @@ tr50:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1259,14 +1260,14 @@ tr50:
   }
 	goto st22;
 tr56:
-#line 322 "unicorn_http.rl"
-	{MARK(start.query, p); }
 #line 323 "unicorn_http.rl"
+	{MARK(start.query, p); }
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1284,12 +1285,12 @@ tr56:
   }
 	goto st22;
 tr60:
-#line 323 "unicorn_http.rl"
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -1310,7 +1311,7 @@ st22:
 	if ( ++p == pe )
 		goto _test_eof22;
 case 22:
-#line 1314 "unicorn_http.c"
+#line 1315 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr42;
 		case 35: goto st0;
@@ -1321,14 +1322,14 @@ case 22:
 		goto st0;
 	goto tr41;
 tr41:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st23;
 st23:
 	if ( ++p == pe )
 		goto _test_eof23;
 case 23:
-#line 1332 "unicorn_http.c"
+#line 1333 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr45;
 		case 35: goto st0;
@@ -1339,14 +1340,14 @@ case 23:
 		goto st0;
 	goto st23;
 tr43:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st24;
 st24:
 	if ( ++p == pe )
 		goto _test_eof24;
 case 24:
-#line 1350 "unicorn_http.c"
+#line 1351 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st25;
@@ -1370,20 +1371,20 @@ case 25:
 		goto st23;
 	goto st0;
 tr6:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st26;
 tr76:
-#line 302 "unicorn_http.rl"
+#line 303 "unicorn_http.rl"
 	{ rb_hash_aset(hp->env, g_http_host, STR_NEW(mark, p)); }
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st26;
 st26:
 	if ( ++p == pe )
 		goto _test_eof26;
 case 26:
-#line 1387 "unicorn_http.c"
+#line 1388 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr49;
 		case 35: goto tr50;
@@ -1421,7 +1422,7 @@ case 28:
 		goto st26;
 	goto st0;
 tr52:
-#line 328 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -1437,7 +1438,7 @@ st29:
 	if ( ++p == pe )
 		goto _test_eof29;
 case 29:
-#line 1441 "unicorn_http.c"
+#line 1442 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr55;
 		case 35: goto tr56;
@@ -1448,14 +1449,14 @@ case 29:
 		goto st0;
 	goto tr54;
 tr54:
-#line 322 "unicorn_http.rl"
+#line 323 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st30;
 st30:
 	if ( ++p == pe )
 		goto _test_eof30;
 case 30:
-#line 1459 "unicorn_http.c"
+#line 1460 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr59;
 		case 35: goto tr60;
@@ -1466,14 +1467,14 @@ case 30:
 		goto st0;
 	goto st30;
 tr57:
-#line 322 "unicorn_http.rl"
+#line 323 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st31;
 st31:
 	if ( ++p == pe )
 		goto _test_eof31;
 case 31:
-#line 1477 "unicorn_http.c"
+#line 1478 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st32;
@@ -1497,58 +1498,58 @@ case 32:
 		goto st30;
 	goto st0;
 tr7:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st33;
 st33:
 	if ( ++p == pe )
 		goto _test_eof33;
 case 33:
-#line 1510 "unicorn_http.c"
+#line 1511 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr63;
 		case 116: goto tr63;
 	}
 	goto st0;
 tr63:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st34;
 st34:
 	if ( ++p == pe )
 		goto _test_eof34;
 case 34:
-#line 1524 "unicorn_http.c"
+#line 1525 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr64;
 		case 116: goto tr64;
 	}
 	goto st0;
 tr64:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st35;
 st35:
 	if ( ++p == pe )
 		goto _test_eof35;
 case 35:
-#line 1538 "unicorn_http.c"
+#line 1539 "unicorn_http.c"
 	switch( (*p) ) {
 		case 80: goto tr65;
 		case 112: goto tr65;
 	}
 	goto st0;
 tr65:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st36;
 st36:
 	if ( ++p == pe )
 		goto _test_eof36;
 case 36:
-#line 1552 "unicorn_http.c"
+#line 1553 "unicorn_http.c"
 	switch( (*p) ) {
 		case 58: goto tr66;
 		case 83: goto tr67;
@@ -1556,7 +1557,7 @@ case 36:
 	}
 	goto st0;
 tr66:
-#line 299 "unicorn_http.rl"
+#line 300 "unicorn_http.rl"
 	{
     rb_hash_aset(hp->env, g_rack_url_scheme, STR_NEW(mark, p));
   }
@@ -1565,7 +1566,7 @@ st37:
 	if ( ++p == pe )
 		goto _test_eof37;
 case 37:
-#line 1569 "unicorn_http.c"
+#line 1570 "unicorn_http.c"
 	if ( (*p) == 47 )
 		goto st38;
 	goto st0;
@@ -1653,14 +1654,14 @@ case 42:
 		goto st40;
 	goto st0;
 tr72:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st43;
 st43:
 	if ( ++p == pe )
 		goto _test_eof43;
 case 43:
-#line 1664 "unicorn_http.c"
+#line 1665 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st41;
 		case 47: goto tr76;
@@ -1712,14 +1713,14 @@ case 44:
 		goto st0;
 	goto st40;
 tr73:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st45;
 st45:
 	if ( ++p == pe )
 		goto _test_eof45;
 case 45:
-#line 1723 "unicorn_http.c"
+#line 1724 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st41;
 		case 47: goto st0;
@@ -1797,14 +1798,14 @@ case 47:
 		goto st0;
 	goto st40;
 tr67:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st48;
 st48:
 	if ( ++p == pe )
 		goto _test_eof48;
 case 48:
-#line 1808 "unicorn_http.c"
+#line 1809 "unicorn_http.c"
 	if ( (*p) == 58 )
 		goto tr66;
 	goto st0;
@@ -2320,14 +2321,14 @@ case 67:
 		goto tr3;
 	goto st0;
 tr2:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st68;
 st68:
 	if ( ++p == pe )
 		goto _test_eof68;
 case 68:
-#line 2331 "unicorn_http.c"
+#line 2332 "unicorn_http.c"
 	switch( (*p) ) {
 		case 32: goto tr3;
 		case 33: goto st49;
@@ -2411,14 +2412,14 @@ case 70:
 		goto st51;
 	goto st0;
 tr100:
-#line 298 "unicorn_http.rl"
+#line 299 "unicorn_http.rl"
 	{ request_method(hp, PTR_TO(mark), LEN(mark, p)); }
 	goto st71;
 st71:
 	if ( ++p == pe )
 		goto _test_eof71;
 case 71:
-#line 2422 "unicorn_http.c"
+#line 2423 "unicorn_http.c"
 	switch( (*p) ) {
 		case 42: goto tr101;
 		case 47: goto tr102;
@@ -2427,14 +2428,14 @@ case 71:
 	}
 	goto st0;
 tr101:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st72;
 st72:
 	if ( ++p == pe )
 		goto _test_eof72;
 case 72:
-#line 2438 "unicorn_http.c"
+#line 2439 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr104;
 		case 13: goto tr105;
@@ -2443,7 +2444,7 @@ case 72:
 	}
 	goto st0;
 tr106:
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2461,7 +2462,7 @@ tr106:
   }
 	goto st73;
 tr119:
-#line 328 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -2472,7 +2473,7 @@ tr119:
     if (!STR_CSTR_EQ(val, "*"))
       rb_hash_aset(hp->env, g_path_info, val);
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2490,14 +2491,14 @@ tr119:
   }
 	goto st73;
 tr126:
-#line 322 "unicorn_http.rl"
-	{MARK(start.query, p); }
 #line 323 "unicorn_http.rl"
+	{MARK(start.query, p); }
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2515,12 +2516,12 @@ tr126:
   }
 	goto st73;
 tr131:
-#line 323 "unicorn_http.rl"
+#line 324 "unicorn_http.rl"
 	{
     VALIDATE_MAX_URI_LENGTH(LEN(start.query, p), QUERY_STRING);
     rb_hash_aset(hp->env, g_query_string, STR_NEW(start.query, p));
   }
-#line 303 "unicorn_http.rl"
+#line 304 "unicorn_http.rl"
 	{
     VALUE str;
 
@@ -2541,7 +2542,7 @@ st73:
 	if ( ++p == pe )
 		goto _test_eof73;
 case 73:
-#line 2545 "unicorn_http.c"
+#line 2546 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr108;
 		case 13: goto tr109;
@@ -2554,14 +2555,14 @@ case 73:
 		goto st0;
 	goto tr107;
 tr107:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st74;
 st74:
 	if ( ++p == pe )
 		goto _test_eof74;
 case 74:
-#line 2565 "unicorn_http.c"
+#line 2566 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr112;
 		case 13: goto tr113;
@@ -2574,14 +2575,14 @@ case 74:
 		goto st0;
 	goto st74;
 tr110:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st75;
 st75:
 	if ( ++p == pe )
 		goto _test_eof75;
 case 75:
-#line 2585 "unicorn_http.c"
+#line 2586 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st76;
@@ -2605,20 +2606,20 @@ case 76:
 		goto st74;
 	goto st0;
 tr102:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st77;
 tr147:
-#line 302 "unicorn_http.rl"
+#line 303 "unicorn_http.rl"
 	{ rb_hash_aset(hp->env, g_http_host, STR_NEW(mark, p)); }
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st77;
 st77:
 	if ( ++p == pe )
 		goto _test_eof77;
 case 77:
-#line 2622 "unicorn_http.c"
+#line 2623 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr117;
 		case 13: goto tr118;
@@ -2658,7 +2659,7 @@ case 79:
 		goto st77;
 	goto st0;
 tr121:
-#line 328 "unicorn_http.rl"
+#line 329 "unicorn_http.rl"
 	{
     VALUE val;
 
@@ -2674,7 +2675,7 @@ st80:
 	if ( ++p == pe )
 		goto _test_eof80;
 case 80:
-#line 2678 "unicorn_http.c"
+#line 2679 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr124;
 		case 13: goto tr125;
@@ -2687,14 +2688,14 @@ case 80:
 		goto st0;
 	goto tr123;
 tr123:
-#line 322 "unicorn_http.rl"
+#line 323 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st81;
 st81:
 	if ( ++p == pe )
 		goto _test_eof81;
 case 81:
-#line 2698 "unicorn_http.c"
+#line 2699 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr129;
 		case 13: goto tr130;
@@ -2707,14 +2708,14 @@ case 81:
 		goto st0;
 	goto st81;
 tr127:
-#line 322 "unicorn_http.rl"
+#line 323 "unicorn_http.rl"
 	{MARK(start.query, p); }
 	goto st82;
 st82:
 	if ( ++p == pe )
 		goto _test_eof82;
 case 82:
-#line 2718 "unicorn_http.c"
+#line 2719 "unicorn_http.c"
 	if ( (*p) < 65 ) {
 		if ( 48 <= (*p) && (*p) <= 57 )
 			goto st83;
@@ -2738,58 +2739,58 @@ case 83:
 		goto st81;
 	goto st0;
 tr103:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st84;
 st84:
 	if ( ++p == pe )
 		goto _test_eof84;
 case 84:
-#line 2751 "unicorn_http.c"
+#line 2752 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr134;
 		case 116: goto tr134;
 	}
 	goto st0;
 tr134:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st85;
 st85:
 	if ( ++p == pe )
 		goto _test_eof85;
 case 85:
-#line 2765 "unicorn_http.c"
+#line 2766 "unicorn_http.c"
 	switch( (*p) ) {
 		case 84: goto tr135;
 		case 116: goto tr135;
 	}
 	goto st0;
 tr135:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st86;
 st86:
 	if ( ++p == pe )
 		goto _test_eof86;
 case 86:
-#line 2779 "unicorn_http.c"
+#line 2780 "unicorn_http.c"
 	switch( (*p) ) {
 		case 80: goto tr136;
 		case 112: goto tr136;
 	}
 	goto st0;
 tr136:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st87;
 st87:
 	if ( ++p == pe )
 		goto _test_eof87;
 case 87:
-#line 2793 "unicorn_http.c"
+#line 2794 "unicorn_http.c"
 	switch( (*p) ) {
 		case 58: goto tr137;
 		case 83: goto tr138;
@@ -2797,7 +2798,7 @@ case 87:
 	}
 	goto st0;
 tr137:
-#line 299 "unicorn_http.rl"
+#line 300 "unicorn_http.rl"
 	{
     rb_hash_aset(hp->env, g_rack_url_scheme, STR_NEW(mark, p));
   }
@@ -2806,7 +2807,7 @@ st88:
 	if ( ++p == pe )
 		goto _test_eof88;
 case 88:
-#line 2810 "unicorn_http.c"
+#line 2811 "unicorn_http.c"
 	if ( (*p) == 47 )
 		goto st89;
 	goto st0;
@@ -2894,14 +2895,14 @@ case 93:
 		goto st91;
 	goto st0;
 tr143:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st94;
 st94:
 	if ( ++p == pe )
 		goto _test_eof94;
 case 94:
-#line 2905 "unicorn_http.c"
+#line 2906 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st92;
 		case 47: goto tr147;
@@ -2953,14 +2954,14 @@ case 95:
 		goto st0;
 	goto st91;
 tr144:
-#line 289 "unicorn_http.rl"
+#line 290 "unicorn_http.rl"
 	{MARK(mark, p); }
 	goto st96;
 st96:
 	if ( ++p == pe )
 		goto _test_eof96;
 case 96:
-#line 2964 "unicorn_http.c"
+#line 2965 "unicorn_http.c"
 	switch( (*p) ) {
 		case 37: goto st92;
 		case 47: goto st0;
@@ -3038,14 +3039,14 @@ case 98:
 		goto st0;
 	goto st91;
 tr138:
-#line 293 "unicorn_http.rl"
+#line 294 "unicorn_http.rl"
 	{ downcase_char(deconst(p)); }
 	goto st99;
 st99:
 	if ( ++p == pe )
 		goto _test_eof99;
 case 99:
-#line 3049 "unicorn_http.c"
+#line 3050 "unicorn_http.c"
 	if ( (*p) == 58 )
 		goto tr137;
 	goto st0;
@@ -3065,7 +3066,7 @@ case 100:
 		goto tr152;
 	goto st0;
 tr151:
-#line 338 "unicorn_http.rl"
+#line 339 "unicorn_http.rl"
 	{
     hp->len.chunk = step_incr(hp->len.chunk, (*p), 16);
     if (hp->len.chunk < 0)
@@ -3076,7 +3077,7 @@ st101:
 	if ( ++p == pe )
 		goto _test_eof101;
 case 101:
-#line 3080 "unicorn_http.c"
+#line 3081 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr153;
 		case 13: goto st102;
@@ -3093,7 +3094,7 @@ case 101:
 		goto tr152;
 	goto st0;
 tr153:
-#line 367 "unicorn_http.rl"
+#line 368 "unicorn_http.rl"
 	{
     HP_FL_SET(hp, INTRAILER);
     cs = http_parser_en_Trailers;
@@ -3106,7 +3107,7 @@ st123:
 	if ( ++p == pe )
 		goto _test_eof123;
 case 123:
-#line 3110 "unicorn_http.c"
+#line 3111 "unicorn_http.c"
 	goto st0;
 st102:
 	if ( ++p == pe )
@@ -3116,7 +3117,7 @@ case 102:
 		goto tr153;
 	goto st0;
 tr152:
-#line 338 "unicorn_http.rl"
+#line 339 "unicorn_http.rl"
 	{
     hp->len.chunk = step_incr(hp->len.chunk, (*p), 16);
     if (hp->len.chunk < 0)
@@ -3127,7 +3128,7 @@ st103:
 	if ( ++p == pe )
 		goto _test_eof103;
 case 103:
-#line 3131 "unicorn_http.c"
+#line 3132 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto st104;
 		case 13: goto st107;
@@ -3148,7 +3149,7 @@ st104:
 case 104:
 	goto tr159;
 tr159:
-#line 375 "unicorn_http.rl"
+#line 376 "unicorn_http.rl"
 	{
   skip_chunk_data_hack: {
     size_t nr = MIN((size_t)hp->len.chunk, REMAINING);
@@ -3170,7 +3171,7 @@ st105:
 	if ( ++p == pe )
 		goto _test_eof105;
 case 105:
-#line 3174 "unicorn_http.c"
+#line 3175 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto st100;
 		case 13: goto st106;
@@ -3377,30 +3378,30 @@ case 113:
 		goto st113;
 	goto st0;
 tr172:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st114;
 tr175:
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st114;
 tr182:
-#line 295 "unicorn_http.rl"
-	{ MARK(mark, p); }
 #line 296 "unicorn_http.rl"
+	{ MARK(mark, p); }
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st114;
 tr185:
-#line 296 "unicorn_http.rl"
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st114;
 st114:
 	if ( ++p == pe )
 		goto _test_eof114;
 case 114:
-#line 3404 "unicorn_http.c"
+#line 3405 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto st115;
 		case 10: goto tr167;
@@ -3429,14 +3430,14 @@ case 114:
 		goto tr169;
 	goto st0;
 tr171:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st115;
 st115:
 	if ( ++p == pe )
 		goto _test_eof115;
 case 115:
-#line 3440 "unicorn_http.c"
+#line 3441 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr171;
 		case 10: goto tr172;
@@ -3448,14 +3449,14 @@ case 115:
 		goto st0;
 	goto tr170;
 tr170:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st116;
 st116:
 	if ( ++p == pe )
 		goto _test_eof116;
 case 116:
-#line 3459 "unicorn_http.c"
+#line 3460 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr175;
 		case 13: goto tr176;
@@ -3468,35 +3469,35 @@ case 116:
 		goto st0;
 	goto st116;
 tr173:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st117;
 tr176:
-#line 297 "unicorn_http.rl"
+#line 298 "unicorn_http.rl"
 	{ write_cont_value(hp, buffer, p); }
 	goto st117;
 tr183:
-#line 295 "unicorn_http.rl"
-	{ MARK(mark, p); }
 #line 296 "unicorn_http.rl"
+	{ MARK(mark, p); }
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st117;
 tr186:
-#line 296 "unicorn_http.rl"
+#line 297 "unicorn_http.rl"
 	{ write_value(hp, buffer, p); }
 	goto st117;
 st117:
 	if ( ++p == pe )
 		goto _test_eof117;
 case 117:
-#line 3495 "unicorn_http.c"
+#line 3496 "unicorn_http.c"
 	if ( (*p) == 10 )
 		goto st114;
 	goto st0;
 tr167:
-#line 362 "unicorn_http.rl"
+#line 363 "unicorn_http.rl"
 	{
     cs = http_parser_first_final;
     goto post_exec;
@@ -3506,7 +3507,7 @@ st124:
 	if ( ++p == pe )
 		goto _test_eof124;
 case 124:
-#line 3510 "unicorn_http.c"
+#line 3511 "unicorn_http.c"
 	goto st0;
 st118:
 	if ( ++p == pe )
@@ -3516,20 +3517,20 @@ case 118:
 		goto tr167;
 	goto st0;
 tr169:
-#line 291 "unicorn_http.rl"
-	{ MARK(start.field, p); }
 #line 292 "unicorn_http.rl"
+	{ MARK(start.field, p); }
+#line 293 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st119;
 tr178:
-#line 292 "unicorn_http.rl"
+#line 293 "unicorn_http.rl"
 	{ snake_upcase_char(deconst(p)); }
 	goto st119;
 st119:
 	if ( ++p == pe )
 		goto _test_eof119;
 case 119:
-#line 3533 "unicorn_http.c"
+#line 3534 "unicorn_http.c"
 	switch( (*p) ) {
 		case 33: goto tr178;
 		case 58: goto tr179;
@@ -3555,18 +3556,18 @@ case 119:
 		goto tr178;
 	goto st0;
 tr181:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st120;
 tr179:
-#line 294 "unicorn_http.rl"
+#line 295 "unicorn_http.rl"
 	{ hp->s.field_len = LEN(start.field, p); }
 	goto st120;
 st120:
 	if ( ++p == pe )
 		goto _test_eof120;
 case 120:
-#line 3570 "unicorn_http.c"
+#line 3571 "unicorn_http.c"
 	switch( (*p) ) {
 		case 9: goto tr181;
 		case 10: goto tr182;
@@ -3578,14 +3579,14 @@ case 120:
 		goto st0;
 	goto tr180;
 tr180:
-#line 295 "unicorn_http.rl"
+#line 296 "unicorn_http.rl"
 	{ MARK(mark, p); }
 	goto st121;
 st121:
 	if ( ++p == pe )
 		goto _test_eof121;
 case 121:
-#line 3589 "unicorn_http.c"
+#line 3590 "unicorn_http.c"
 	switch( (*p) ) {
 		case 10: goto tr185;
 		case 13: goto tr186;
@@ -3726,7 +3727,7 @@ case 121:
 	_out: {}
 	}
 
-#line 436 "unicorn_http.rl"
+#line 437 "unicorn_http.rl"
 post_exec: /* "_out:" also goes here */
   if (hp->cs != http_parser_error)
     hp->cs = cs;
@@ -3901,6 +3902,10 @@ static VALUE HttpParser_clear(VALUE self)
 {
   struct http_parser *hp = data_get(self);
 
+  /* we can't safely reuse .buf and .env if hijacked */
+  if (HP_FL_TEST(hp, HIJACK))
+    return HttpParser_init(self);
+
   http_parser_init(hp);
   my_hash_clear(hp->env);
 
@@ -4107,6 +4112,15 @@ static VALUE HttpParser_env(VALUE self)
   return data_get(self)->env;
 }
 
+static VALUE HttpParser_hijacked_bang(VALUE self)
+{
+  struct http_parser *hp = data_get(self);
+
+  HP_FL_SET(hp, HIJACK);
+
+  return self;
+}
+
 /**
  * call-seq:
  *    parser.filter_body(dst, src) => nil/src
@@ -4211,8 +4225,11 @@ static VALUE HttpParser_rssget(VALUE self)
 
 void Init_unicorn_http(void)
 {
+  static VALUE mark_ary;
   VALUE mUnicorn, cHttpParser;
 
+  mark_ary = rb_ary_new();
+  rb_global_variable(&mark_ary);
   mUnicorn = rb_define_module("Unicorn");
   cHttpParser = rb_define_class_under(mUnicorn, "HttpParser", rb_cObject);
   eHttpParserError =
@@ -4222,7 +4239,7 @@ void Init_unicorn_http(void)
   e414 = rb_define_class_under(mUnicorn, "RequestURITooLongError",
                                eHttpParserError);
 
-  init_globals();
+  init_globals(mark_ary);
   rb_define_alloc_func(cHttpParser, HttpParser_alloc);
   rb_define_method(cHttpParser, "initialize", HttpParser_init, 0);
   rb_define_method(cHttpParser, "clear", HttpParser_clear, 0);
@@ -4238,6 +4255,7 @@ void Init_unicorn_http(void)
   rb_define_method(cHttpParser, "next?", HttpParser_next, 0);
   rb_define_method(cHttpParser, "buf", HttpParser_buf, 0);
   rb_define_method(cHttpParser, "env", HttpParser_env, 0);
+  rb_define_method(cHttpParser, "hijacked!", HttpParser_hijacked_bang, 0);
   rb_define_method(cHttpParser, "response_start_sent=", HttpParser_rssset, 1);
   rb_define_method(cHttpParser, "response_start_sent", HttpParser_rssget, 0);
 
@@ -4258,14 +4276,16 @@ void Init_unicorn_http(void)
 
   rb_define_singleton_method(cHttpParser, "max_header_len=", set_maxhdrlen, 1);
 
-  init_common_fields();
+  init_common_fields(mark_ary);
   SET_GLOBAL(g_http_host, "HOST");
   SET_GLOBAL(g_http_trailer, "TRAILER");
   SET_GLOBAL(g_http_transfer_encoding, "TRANSFER_ENCODING");
   SET_GLOBAL(g_content_length, "CONTENT_LENGTH");
   SET_GLOBAL(g_http_connection, "CONNECTION");
   id_set_backtrace = rb_intern("set_backtrace");
-  init_unicorn_httpdate();
+  init_unicorn_httpdate(mark_ary);
+
+  OBJ_FREEZE(mark_ary);
 
 #ifndef HAVE_RB_HASH_CLEAR
   id_clear = rb_intern("clear");
